==> castle_api-0.5.3.crate: castle_api-0.5.3/src/lib.rs <==
#![feature(if_let_guard)]
use std::future::Future;
use std::{fmt::Debug};
pub use castle_query_parser::Input;
pub use castle_query_parser::{Field, Inputs, Projection};
pub use types::value::Value;

pub use crate::castle::Castle;
pub use castle_tokenizer::{Primitive, Number};

mod validation;
pub mod castle;
pub mod types;

impl<Ctx, E> PartialEq for dyn Resolver<Ctx, E> + {
    fn eq(&self, other: &Self) -> bool {
        self as *const _ == other as *const _
    }
}

impl<Ctx, E> Debug for dyn Resolver<Ctx, E> + {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "Resolver {}", std::any::type_name::<Self>())
    }
}


//A resolver takes in fields (inner wants), arguments and context and returns the resolved want
#[async_trait::async_trait]
pub trait Resolver<Ctx, E>: Send + Sync {
    async fn resolve(&self, field: &Field, ctx: &Ctx) -> Result<Value<Ctx, E>, E>;
}

// This allows closures to impl the resolver trait
#[async_trait::async_trait]
impl<Ctx, F, E, Fut> Resolver<Ctx, E> for F
where
    Ctx: Debug + Sync + Send,
    F: Fn(&Field, &Ctx) -> Fut + Send + Sync,
    Fut: Future<Output = Result<Value<Ctx, E>, E>> + Send,
{
    async fn resolve(&self, field: &Field, ctx: &Ctx) -> Result<Value<Ctx, E>, E> {
        self(field, ctx).await
    }
}


#[async_trait::async_trait]
pub trait Directive<Ctx: Send + 'static, E: 'static>: Send + Sync {
    async fn field_visitor(&self, _field: &Field, _directive_args: &Inputs, _value: Box<dyn Resolver<Ctx, E>>, _context: Ctx) -> Result<Value<Ctx, E>, E> {
        unimplemented!()
    }
}

==> castle_api-0.5.4.crate: castle_api-0.5.4/src/lib.rs <==
#![feature(if_let_guard)]
use std::future::Future;
use std::{fmt::Debug};
pub use castle_query_parser::Input;
pub use castle_query_parser::{Field, Inputs, Projection};
pub use types::value::Value;

pub use crate::castle::Castle;
pub use castle_tokenizer::{Primitive, Number};

mod validation;
pub mod castle;
pub mod types;

impl<Ctx, E> PartialEq for dyn Resolver<Ctx, E> + {
    fn eq(&self, other: &Self) -> bool {
        self as *const _ == other as *const _
    }
}

impl<Ctx, E> Debug for dyn Resolver<Ctx, E> + {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "Resolver {}", std::any::type_name::<Self>())
    }
}


//A resolver takes in fields (inner wants), arguments and context and returns the resolved want
#[async_trait::async_trait]
pub trait Resolver<Ctx, E>: Send + Sync {
    async fn resolve(&self, field: &Field, ctx: &Ctx) -> Result<Value<Ctx, E>, E>;
}

/// ### Allows async closures to `impl` the [Resolver] trait.
///
/// This is ***dark magic***. Don't ask how it works I won't be able to explain it.
/// This is needed in order to make any async functions work
/// with the add_resolver function.
trait Fn2Args<Arg1, Arg2>: Fn(Arg1, Arg2) -> <Self as Fn2Args<Arg1, Arg2>>::Output {
    type Output;
}
impl<F: Fn(Arg1, Arg2) -> Out, Arg1, Arg2, Out> Fn2Args<Arg1, Arg2> for F {
    type Output = Out;
}

#[async_trait::async_trait]
impl<F, Ctx: Send + Sync, E: Send + Sync> Resolver<Ctx, E> for F
where
    F: for<'a, 'b> Fn2Args<&'a Field, &'b Ctx> + Sync + Send,
    for<'a, 'b> <F as Fn2Args<&'a Field, &'b Ctx>>::Output: Future<Output = Result<Value<Ctx, E>, E>> + Send,
{
    async fn resolve(&self, field: &Field, ctx: &Ctx) -> Result<Value<Ctx, E>, E> {
        self(field, ctx).await
    }
}


#[async_trait::async_trait]
pub trait Directive<Ctx: Send + 'static, E: 'static>: Send + Sync {
    async fn field_visitor(&self, _field: &Field, _directive_args: &Inputs, _value: Box<dyn Resolver<Ctx, E>>, _context: Ctx) -> Result<Value<Ctx, E>, E> {
        unimplemented!()
    }
}
